@@ -653,8 +653,7 @@ select_auth_authenticate (SoupSession *session, SoupMessage *msg,
 }
 
 static void
-select_auth_test_one (SoupURI *uri,
-		      gboolean disable_digest, const char *password,
+select_auth_test_one (SoupURI *uri, const char *password,
 		      const char *first_headers, const char *first_response,
 		      const char *second_headers, const char *second_response,
 		      guint final_status)
@@ -664,9 +663,6 @@ select_auth_test_one (SoupURI *uri,
 	SoupSession *session;
 
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	if (disable_digest)
-		soup_session_remove_feature_by_type (session, SOUP_TYPE_AUTH_DIGEST);
-
 	g_signal_connect (session, "authenticate",
 			  G_CALLBACK (select_auth_authenticate), &sad);
 	memset (&sad, 0, sizeof (sad));
@@ -730,9 +726,7 @@ static gboolean
 server_basic_auth_callback (SoupAuthDomain *auth_domain, SoupMessage *msg,
 			    const char *username, const char *password, gpointer data)
 {
-	if (strcmp (username, "user") != 0)
-		return FALSE;
-	return strcmp (password, "good-basic") == 0;
+	return FALSE;
 }
 
 static char *
@@ -782,43 +776,28 @@ do_select_auth_test (void)
 		NULL);
 	soup_server_add_auth_domain (server, digest_auth_domain);
 
+	/* FIXME: when we support disabling auth types in the session,
+	 * test that too.
+	 */
+
 	debug_printf (1, "  Testing with no auth\n");
-	select_auth_test_one (uri, FALSE, NULL,
+	select_auth_test_one (uri, NULL,
 			      "Basic, Digest", "Digest",
 			      NULL, NULL,
 			      SOUP_STATUS_UNAUTHORIZED);
 
 	debug_printf (1, "  Testing with bad password\n");
-	select_auth_test_one (uri, FALSE, "bad",
+	select_auth_test_one (uri, "bad",
 			      "Basic, Digest", "Digest",
 			      "Basic, Digest", "Digest",
 			      SOUP_STATUS_UNAUTHORIZED);
 
 	debug_printf (1, "  Testing with good password\n");
-	select_auth_test_one (uri, FALSE, "good",
+	select_auth_test_one (uri, "good",
 			      "Basic, Digest", "Digest",
 			      NULL, NULL,
 			      SOUP_STATUS_OK);
 
-	/* Test with Digest disabled in the client. */
-	debug_printf (1, "  Testing without Digest with no auth\n");
-	select_auth_test_one (uri, TRUE, NULL,
-			      "Basic, Digest", "Basic",
-			      NULL, NULL,
-			      SOUP_STATUS_UNAUTHORIZED);
-
-	debug_printf (1, "  Testing without Digest with bad password\n");
-	select_auth_test_one (uri, TRUE, "bad",
-			      "Basic, Digest", "Basic",
-			      "Basic, Digest", "Basic",
-			      SOUP_STATUS_UNAUTHORIZED);
-
-	debug_printf (1, "  Testing without Digest with good password\n");
-	select_auth_test_one (uri, TRUE, "good-basic",
-			      "Basic, Digest", "Basic",
-			      NULL, NULL,
-			      SOUP_STATUS_OK);
-
 	/* Now flip the order of the domains, verify that this flips
 	 * the order of the headers, and make sure that digest auth
 	 * *still* gets used.
@@ -830,19 +809,19 @@ do_select_auth_test (void)
 	soup_server_add_auth_domain (server, basic_auth_domain);
 
 	debug_printf (1, "  Testing flipped with no auth\n");
-	select_auth_test_one (uri, FALSE, NULL,
+	select_auth_test_one (uri, NULL,
 			      "Digest, Basic", "Digest",
 			      NULL, NULL,
 			      SOUP_STATUS_UNAUTHORIZED);
 
 	debug_printf (1, "  Testing flipped with bad password\n");
-	select_auth_test_one (uri, FALSE, "bad",
+	select_auth_test_one (uri, "bad",
 			      "Digest, Basic", "Digest",
 			      "Digest, Basic", "Digest",
 			      SOUP_STATUS_UNAUTHORIZED);
 
 	debug_printf (1, "  Testing flipped with good password\n");
-	select_auth_test_one (uri, FALSE, "good",
+	select_auth_test_one (uri, "good",
 			      "Digest, Basic", "Digest",
 			      NULL, NULL,
 			      SOUP_STATUS_OK);
