@@ -22,8 +22,8 @@
 #include "test-utils.h"
 
 static struct {
-	char *title, *name;
-	char *result;
+	const char *title, *name;
+	const char *result;
 } tests[] = {
 	/* Both fields must be filled in */
 	{ NULL, "Name", "" },
@@ -158,8 +158,13 @@ do_md5_test_curl (const char *uri, const char *file, const char *md5)
 	g_free (file_arg);
 }
 
+#define MD5_TEST_FILE SRCDIR "/resources/home.gif"
+#define MD5_TEST_FILE_BASENAME "home.gif"
+#define MD5_TEST_FILE_MIME_TYPE "image/gif"
+
 static void
-do_md5_test_libsoup (const char *uri, const char *contents, const char *md5)
+do_md5_test_libsoup (const char *uri, const char *contents,
+		     gsize length, const char *md5)
 {
 	SoupMultipart *multipart;
 	SoupBuffer *buffer;
@@ -169,9 +174,10 @@ do_md5_test_libsoup (const char *uri, const char *contents, const char *md5)
 	debug_printf (1, "  via libsoup: ");
 
 	multipart = soup_multipart_new (SOUP_FORM_MIME_TYPE_MULTIPART);
-	buffer = soup_buffer_new (SOUP_MEMORY_COPY, contents, strlen (contents));
+	buffer = soup_buffer_new (SOUP_MEMORY_COPY, contents, length);
 	soup_multipart_append_form_file (multipart, "file",
-					 "index.txt", "text/plain",
+					 MD5_TEST_FILE_BASENAME,
+					 MD5_TEST_FILE_MIME_TYPE,
 					 buffer);
 	soup_buffer_free (buffer);
 	soup_multipart_append_form_string (multipart, "fmt", "text");
@@ -206,8 +212,8 @@ do_md5_tests (const char *uri)
 
 	debug_printf (1, "\nMD5 tests (POST, multipart/form-data)\n");
 
-	if (!g_file_get_contents (SRCDIR "/index.txt", &contents, &length, &error)) {
-		debug_printf (1, "  ERROR: Could not read " SRCDIR "/index.txt: %s\n", error->message);
+	if (!g_file_get_contents (MD5_TEST_FILE, &contents, &length, &error)) {
+		debug_printf (1, "  ERROR: Could not read " MD5_TEST_FILE ": %s\n", error->message);
 		g_error_free (error);
 		errors++;
 		return;
@@ -215,8 +221,8 @@ do_md5_tests (const char *uri)
 
 	md5 = g_compute_checksum_for_string (G_CHECKSUM_MD5, contents, length);
 
-	do_md5_test_curl (uri, SRCDIR "/index.txt", md5);
-	do_md5_test_libsoup (uri, contents, md5);
+	do_md5_test_curl (uri, MD5_TEST_FILE, md5);
+	do_md5_test_libsoup (uri, contents, length, md5);
 
 	g_free (contents);
 	g_free (md5);
@@ -412,11 +418,11 @@ main (int argc, char **argv)
 	loop = g_main_loop_new (NULL, TRUE);
 
 	if (run_tests) {
-		uri_str = g_strdup_printf ("http://localhost:%u/hello", port);
+		uri_str = g_strdup_printf ("http://127.0.0.1:%u/hello", port);
 		do_hello_tests (uri_str);
 		g_free (uri_str);
 
-		uri_str = g_strdup_printf ("http://localhost:%u/md5", port);
+		uri_str = g_strdup_printf ("http://127.0.0.1:%u/md5", port);
 		do_md5_tests (uri_str);
 		g_free (uri_str);
 	} else {
@@ -426,6 +432,7 @@ main (int argc, char **argv)
 
 	g_main_loop_unref (loop);
 
+	soup_test_server_quit_unref (server);
 	if (run_tests)
 		test_cleanup ();
 	return errors != 0;
